'use strict'

const mssql = require('./')

const sqlConfig = {
  password: 'P@ssw0rd',
  database: 'StockDB',
  stream: false,
  options: {
    trustServerCertificate: true,
    enableArithAbort: true,
    encrypt: true
  },
  pool: {
    max: 1,
    min: 0,
  },
  port: 1433,
  user: 'sa',
  server: 'localhost',
}

const client = new mssql.ConnectionPool(sqlConfig)
let runInterval = null

async function run () {
  const pool = await client.connect()
  const request = pool.request()
  console.time('query')
  const clientId = 12
  const startDate = '2018-01-01'
  const endDate = '2020-06-18'
  await request.input('clientId', mssql.Int, clientId)
    .input('startDate', mssql.DateTime, new Date(startDate))
    .input('endDate', mssql.DateTime, new Date(endDate))
    .query(`SELECT v.Id as VehicleId,
                            n.Name as Make, 
                            m.Name as Model, 
                            v.Variant as Variant,
                            v.Registration as Registration,
                            v.Price As Price,
                            v.OfferPrice as OfferPrice,
                            v.OfferDisplay as OfferDisplay,
                            d.Name as DealershipName,
                            v.StockDate as StockDate,
                            v.PurchaseStandIn as SIV,
                            s.Name as PurchaseSource,
                            DATEDIFF(day, StockDate, GETDATE()) as DaysInStock,
                            v.SoldDate as SoldDate,
                            v.SoldPrice as SoldPrice,
                            s1.Name as SoldSource
                    FROM  UsedVehicles v 
                    JOIN  Models m on m.Id = v.ModelId
                    JOIN  Makes n ON n.Id = m.MakeId
                    JOIN  Dealerships d on d.Id = v.DealershipId
                    LEFT JOIN UsedPriceSources s on s.Id = v.PurchasePriceSourceId
                    LEFT JOIN UsedPriceSources s1 on s1.Id = v.SoldPriceSourceId
                    WHERE v.Status = 3
                    AND v.ClientId = @clientId
                    AND v.SoldDate between @startDate AND @endDate
                    ORDER BY SoldDate ASC`)
  console.timeEnd('query')
  // console.log(result)
  return client.close()
}

run()
